package geniusweb.actions;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;

import javax.swing.JFileChooser;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileNameExtensionFilter;

import geniusweb.domaineditor.ProfileEditor;
import geniusweb.domaineditor.model.profile.LinearAdditiveUtilitySpaceModel;
import geniusweb.profile.utilityspace.LinearAdditiveUtilitySpace;
import tudelft.utilities.logging.Reporter;

public class SaveAction extends GuiAction {

	public SaveAction(Reporter log) {
		super("Save", log);
		putValue(SHORT_DESCRIPTION, "Save the profile");
		putValue(ACCELERATOR_KEY,
				KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		save(ProfileEditor.getInstance().getModel());
	}

	/**
	 * Save a model.
	 * 
	 * @param model the {@link LinearAdditiveUtilitySpaceModel} to be saved
	 */
	protected void save(LinearAdditiveUtilitySpaceModel model) {
		LinearAdditiveUtilitySpace profile = model.getCurrentValue();
		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(new File(profile.getName() + ".json"));
		FileNameExtensionFilter filter = new FileNameExtensionFilter(
				"JSON file", "json");
		chooser.setFileFilter(filter);
		int selection = chooser.showSaveDialog(null);
		if (selection != JFileChooser.APPROVE_OPTION)
			return;
		File file = chooser.getSelectedFile();
		try {
			jackson.writeValue(file, profile);
		} catch (IOException e) {
			log.log(Level.WARNING, "Failed to save:" + e.getMessage());
			e.printStackTrace();
		}
	}
}
