package geniusweb.domaineditor.panels;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import geniusweb.domaineditor.model.DiscreteValueSetModel;
import geniusweb.domaineditor.model.DomainModel;
import geniusweb.domaineditor.model.NumberValueSetModel;
import geniusweb.domaineditor.model.ValueSetModel;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.mvc.model.MapModel;
import tudelft.utilities.mvc.model.StringModel;
import tudelft.utilities.mvc.panels.CustomMapEditor;
import tudelft.utilities.mvc.panels.LabeledPanel;
import tudelft.utilities.mvc.panels.StringPanel;

/**
 * The actual editor panel for domains Domains are issues and their possible
 * values.
 */
@SuppressWarnings("serial")
public class DomainEditorPanel extends JPanel {

	/**
	 * Panel with on the left a set of strings (issues). When user selects an
	 * issue, an editor is opened for the valueset of that issue.
	 * 
	 * @param domainmodel the {@link DomainModel} containing the issue and
	 *                    valuesets.
	 * @param log         used when generating new issue values
	 */
	public DomainEditorPanel(final DomainModel domainmodel, Reporter log) {
		setLayout(new BorderLayout());

		add(new LabeledPanel("domain name",
				new StringPanel(domainmodel.getNameModel())),
				BorderLayout.NORTH);
		add(new IssuesMapEditor(domainmodel.getIssuesModel(), log),
				BorderLayout.CENTER);

	}
}

/**
 * CustomMapEditor with issues on the left and editor for selected issue on the
 * right.
 */
@SuppressWarnings("serial")
class IssuesMapEditor
		extends CustomMapEditor<String, StringModel, ValueSetModel> {
	private final ValueSetEditorFactory vsetfactory = new ValueSetEditorFactory();
	private Reporter logger;
	private int n = 1;

	public IssuesMapEditor(MapModel<StringModel, ValueSetModel> model,
			Reporter log) {
		super(model, true);
		this.logger = log;
	}

	@Override
	public Component getEditor(StringModel key) {
		if (key == null)
			return null; // how can this be?
		ValueSetModel valuesetmodel = getModel().getValue(key);
		System.out.println("key selected:" + key);
		System.out.println("model contains key:" + valuesetmodel);
		return vsetfactory.create(key, valuesetmodel, logger);
	}

	@Override
	public void addEntry() {
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(new JLabel("Select issue type"), BorderLayout.NORTH);
		String[] types = { "discrete", "number" };
		JComboBox combo = new JComboBox<>(types);
		panel.add(combo, BorderLayout.CENTER);
		JOptionPane.showMessageDialog(this, panel);
		StringModel issuename = new StringModel("issue " + (n++), logger);
		ValueSetModel valuesetmodel;
		if (combo.getSelectedItem().equals("discrete"))
			valuesetmodel = new DiscreteValueSetModel(logger);
		else
			valuesetmodel = new NumberValueSetModel(logger);
		getModel().put(issuename, valuesetmodel);

	}

}
