package geniusweb.domaineditor.panels.profile;

import java.awt.Component;

import javax.swing.JTabbedPane;

import geniusweb.domaineditor.model.profile.DiscreteValueSetUtilitiesModel;
import geniusweb.domaineditor.model.profile.LinearAdditiveUtilitySpaceModel;
import geniusweb.domaineditor.model.profile.NumberValueSetUtilitiesModel;
import geniusweb.domaineditor.model.profile.ValueSetUtilitiesModel;
import geniusweb.domaineditor.panels.BidEditorPanel;
import geniusweb.domaineditor.panels.DomainEditorPanel;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.mvc.model.MapModel;
import tudelft.utilities.mvc.model.StringModel;
import tudelft.utilities.mvc.panels.CustomMapEditor;
import tudelft.utilities.mvc.panels.MapPanel;

/**
 * Editor for entire {@link LinearAdditiveUtilitySpaceModel}
 *
 */
public class ProfileEditorPanel extends JTabbedPane {

	private LinearAdditiveUtilitySpaceModel linmodel;
	private Reporter log;

	public ProfileEditorPanel(LinearAdditiveUtilitySpaceModel lamodel,
			Reporter log) {
		this.linmodel = lamodel;
		this.log = log;

		addTab("Domain", new DomainEditorPanel(linmodel.getDomain(), log));
		addTab("Reservation Bid",
				new BidEditorPanel(linmodel.getReservationBid()));
		addTab("Utilities", new MyUtilsEditor(linmodel.getIssueUtilities()));
		addTab("Weights", new MapPanel<>(linmodel.getWeights()));
	}

}

/**
 * CustomMapEditor for the utilities. Pulled out as getEditor is basically a
 * factory.
 *
 */
class MyUtilsEditor
		extends CustomMapEditor<String, StringModel, ValueSetUtilitiesModel> {

	public MyUtilsEditor(
			MapModel<StringModel, ValueSetUtilitiesModel> issueUtilities) {
		super(issueUtilities, false);
	}

	@Override
	public void addEntry() {
	}

	@Override
	public Component getEditor(StringModel key) {
		ValueSetUtilitiesModel value = getModel().getValue(key);
		if (value instanceof DiscreteValueSetUtilitiesModel)
			return new DiscreteValueSetUtilitiesPanel(
					(DiscreteValueSetUtilitiesModel) value);
		if (value instanceof NumberValueSetUtilitiesModel)
			return new NumberValueSetUtilitiesPanel(
					(NumberValueSetUtilitiesModel) value);
		throw new RuntimeException("Unknown valueset encountered:" + value);
	}

}