package geniusweb.opponentmodel.bayesian;

/**
 * Hypothesis on the weight of an issue.
 * <p>
 * DO NOT USE THIS. Private use for {@link BayesianOpponentModel}.
 */
class WeightHypothesis extends Hypothesis {
	private final double weight;

	public WeightHypothesis(double weight, double p) {
		super(p);
		this.weight = weight;
	}

	public WeightHypothesis withWeight(double value) {
		return new WeightHypothesis(value, getProbability());
	}

	public double getWeight() {
		return weight;
	}

	@Override
	public String toString() {
		return "P(" + weight + ")=" + getProbability();
	}

	@Override
	public WeightHypothesis with(double p) {
		return new WeightHypothesis(weight, p);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(weight);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		WeightHypothesis other = (WeightHypothesis) obj;
		if (Double.doubleToLongBits(weight) != Double
				.doubleToLongBits(other.weight))
			return false;
		return true;
	}
}
